# Copyright 2008, Red Hat, Inc
# Steve 'Ashcrow' Milner <smilner@redhat.com>
#
# This software may be freely redistributed under the terms of the GNU
# general public license.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
Test of director decorators.
"""

__docformat__ = 'restructuredtext'


import exceptions
import inspect
import types
import unittest

from director import decorators


class Fake(object):

    @decorators.simple_help("Test of help")
    def method(self, input):
        return input

    @decorators.general_help(desc="Test of kwhelp",
                             options={'input': 'the input to take'},
                             examples=['app fake method2 somefile.txt'])
    def method2(self, input):
        return input


class ActionTests(unittest.TestCase):
    """
    Tests to verify that base action class works correctly.
    """

    def setUp(self):
        """
        Sets up stuff for the test.
        """
        self.fake_obj = Fake()

    def test_simple_help(self):
        """
        Make sure that the test decorator works as expected.
        """
        self.assertEqual(self.fake_obj.method.help, "Test of help")
        self.assertEqual(type(self.fake_obj.method.meth), types.FunctionType)
        self.assertEqual(inspect.getargspec(self.fake_obj.method.meth)[0],
                         ['self', 'input'])

    def test_general_help(self):
        """
        Make sure that the kwhelp test decorator works as expected.
        """
        rendered = "\nTest of kwhelp\nOPTIONS:\n\tinput\tthe input to take\n\
EXAMPLES:\n\tapp fake method2 somefile.txt"
        self.assertEqual(self.fake_obj.method2.help, rendered)
        self.assertEqual(type(self.fake_obj.method.meth), types.FunctionType)
        self.assertEqual(inspect.getargspec(self.fake_obj.method.meth)[0],
                         ['self', 'input'])



# Copyright 2008, Red Hat, Inc
# Steve 'Ashcrow' Milner <smilner@redhat.com>
#
# This software may be freely redistributed under the terms of the GNU
# general public license.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
Exception filters for director.
"""

__docformat__ = 'restructuredtext'


import exceptions
import sys


class Filter(list):
    """
    Holds all filters to execute on exception.
    """

    def execute_filters(self, exception):
        """
        Executes all filters with an exception.

        :Parameters:
            - `exception`: the exception that is going to be filtered.

        """
        for filter in self:
            filter.filter(exception)

    def register_filter(self, exception_filter):
        """
        Registers an ExceptionFilter.

        :Parameters:
            - `exception_filter`: the ExceptionFilter to add.
        """
        for filter in self:
            if type(exception_filter.exception('')) == \
                    type(filter.exception('')):
                txt = 'You can only have one filter for one exception: %s' % (
                       exception_filter.exception)
                raise Exception(txt)
        self.append(exception_filter)


class ExceptionFilter(object):
    """
    Parent class for all filters.
    """

    def __init__(self, exception, error_text="%s"):
        """
        Creates the ExceptionFilter object.

        :Parameters:
            - `exception`: the exception to filter.
            - `error_text`: the error text to show,
        """
        self.exception = exception
        self.error_text = error_text

    def filter(self, exception):
        """
        Filters an exception. True on filter, False otherwise.

        :Parameters:
            - `exception`: the exception being filtered.
        """
        if type(exception) == type(self.exception('')):
            if "%s" in self.error_text:
                print >> sys.stderr, self.error_text % exception
            else:
                print >> sys.stderr, self.error_text
            return True
        return False



# Copyright 2008, Red Hat, Inc
# Steve 'Ashcrow' Milner <smilner@redhat.com>
#
# This software may be freely redistributed under the terms of the GNU
# general public license.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
Test action to use in unit tests.
"""

__docformat__ = 'restructuredtext'


import director

from director import decorators


class Simpleaction(director.Action):
    """
    Very simple action for use in testing.
    """

    @decorators.simple_help("\nOptions:\topt:\tsome kind of options")
    def verb(self, opt, another=False, last="last"):
        """
        An example verb.
        """
        pass



# Copyright 2008, Red Hat, Inc
# Steve 'Ashcrow' Milner <smilner@redhat.com>
#
# This software may be freely redistributed under the terms of the GNU
# general public license.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
Test of unsupported help styles.
"""

__docformat__ = 'restructuredtext'


import unittest

import director.error

from director import Action


class Fake(Action):

    def method(self, input):
        """
        Doc string.

        == help ==
        this is the help message.
        == end help ==
        """
        return input


class UnsupportedHelpTests(unittest.TestCase):
    """
    Tests to verify that unsupported help doesn't pass through.
    """

    def setUp(self):
        """
        Sets up stuff for the test.
        """
        self.fake_obj = Fake()

    def test_simple_help(self):
        """
        Make sure we get a proper exception.
        """
        #self.assertEqual(self.fake_obj.method.help, "Test of help")
        self.assertRaises(director.error.UnsuportedHelpStyleError,
            self.fake_obj.help, ('method', ))



# Copyright 2008, Red Hat, Inc
# Steve 'Ashcrow' Milner <smilner@redhat.com>
#
# This software may be freely redistributed under the terms of the GNU
# general public license.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
Exception classes.
"""

__docformat__ = 'restructuredtext'


class DirectorError(Exception):
    """
    Base class for Director specific exceptions.
    """
    pass


class UnsuportedHelpStyleError(DirectorError):
    """
    Used when an unsupported help style is attempted.
    """
    pass



# Copyright 2008, Red Hat, Inc
# Steve 'Ashcrow' Milner <smilner@redhat.com>
#
# This software may be freely redistributed under the terms of the GNU
# general public license.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
Code symbols for system related items.

- *SYSTEM_OK*: Exit to shell if everything was OK.
- *NOT_ENOUGH_PARAMETERS*: Exit to shell if not enough parameters are passed.
- *EXCEPTION_RAISED*: Exit to the shell if an exception is raised.
"""

__docformat__ = 'restructuredtext'


SYSTEM_OK = 0
NOT_ENOUGH_PARAMETERS = 1
EXCEPTION_RAISED = 2



# Copyright 2008, Red Hat, Inc
# Steve 'Ashcrow' Milner <smilner@redhat.com>
#
# This software may be freely redistributed under the terms of the GNU
# general public license.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
Tests for filter.
"""

__docformat__ = 'restructuredtext'


import exceptions
import unittest

from director.filter import Filter
from director.filter import ExceptionFilter


class FilterTests(unittest.TestCase):
    """
    Tests the Filter object.
    """

    def setUp(self):
        """
        Sets up stuff for the test.
        """
        self.filter = Filter()

    def tearDown(self):
        """
        Tears down after each test.
        """
        del self.filter

    def test_register_filter(self):
        """
        Tests the register_filter method.
        """
        self.filter.register_filter(ExceptionFilter(exceptions.IOError, 'ERR'))
        self.assertEqual(len(self.filter), 1)

    def test_execute_filters(self):
        """
        Tests the execute_filters method.
        """
        self.filter.execute_filters(exceptions.Exception(''))


class ExceptionFilterTests(unittest.TestCase):
    """
    Tests the code for ExceptionFilters.
    """

    def setUp(self):
        """
        Sets up stuff for the test.
        """
        self.exception_filter = ExceptionFilter(exceptions.IOError)

    def tearDown(self):
        """
        Tears down after each test.
        """
        del self.exception_filter

    def test_filter(self):
        """
        Tests the filter functionality.
        """
        self.exception_filter.filter(exceptions.IOError(''))
        self.exception_filter.filter(exceptions.TypeError(''))



# Copyright 2008, Red Hat, Inc
# Steve 'Ashcrow' Milner <smilner@redhat.com>
#
# This software may be freely redistributed under the terms of the GNU
# general public license.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
Main classes for director.
"""

__docformat__ = 'restructuredtext'
__version__ = '1.2.0'
__license__ = 'GPLv3+'
__author__ = "Steve 'Ashcrow' Milner"


import inspect
import os
import sys
import types
import warnings

import director.error

from optparse import OptionParser

from director import codes
from director.decorators import general_help


class Action(object):
    """
    Base class for command line actions.
    """

    description_txt = "Base action class"

    def __init__(self):
        """
        Create the Action object. Do not override.
        """
        self._startup_hook()

    def __del__(self):
        """
        Work to do when the object is deleted. Do not override.
        """
        self._shutdown_hook()

    def __repr__(self):
        """
        String representation of the object.
        """
        return "<%s Action>" % self.__class__.__name__

    def _startup_hook(self):
        """
        Hook for doing work on creation of the object. It's not in
        __init__ just so that we keep __init__ clean.
        """
        pass

    def _shutdown_hook(self):
        """
        Hook for doing work on deletion of the object. It's not in
        __del__ just so that we keep __del__ clean.
        """
        pass

    def _list_verbs(self):
        """
        Lists all available verbs.
        """
        verbs = []
        for item in dir(self):
            if item[0] != '_':
                if type(self.__getattribute__(item)) == types.MethodType:
                    verbs.append(item)
        return verbs

    def _action_help(self):
        """
        Formats and shows help for all available verbs in an action.
        """
        print >> sys.stderr, "Usage: myapp [verb] [--opt=val]..."
        print >> sys.stderr, "%s\n" % self.description_txt
        for verb in self._list_verbs():
            print >> sys.stderr, "%s -" % verb,
            self.help(verb)
            print >> sys.stderr, "\n"

    @general_help("Detailed help information about the action.",
                  {'verb': 'verb to get help on'},
                  ['myapp list help --verb=help'])
    def help(self, verb=None):
        """
        Detailed help information about the action.

        :Parameters:
            - `verb`: what to get help on.

        Do not override.
        """
        # If we have no verb then give available verbs and a usage message
        if not verb:
            print self.description()
            return

        try:
            print >> sys.stderr, self.__getattribute__(verb).help
        except:
            raise director.error.UnsuportedHelpStyleError(
                'Unsupported help style being used.')

    @general_help("Quick blurb about the action.",
                  examples=['myapp list description'])
    def description(self):
        """
        Quick blurb about the action.

        Do not override. See description_txt
        """
        verbs = ", ".join(self._list_verbs())
        print >> sys.stderr, "%s.\nAvailable verbs: %s" % (
                                      self.description_txt, verbs)
        print >> sys.stderr, "For more detailed usage use myapp \
noun help add --verb=verb."


class ActionRunner(object):
    """
    In charge of running plugins based on information passed in via arguments.
    """

    def __init__(self, args, plugin_package):
        """
        Creates the ActionRunner object.

        :Parameters:
            - `args`: all args passed from command line.
            - `plugin_package`: the name of the package where plugins live.
        """
        self.plugin_package = plugin_package
        self.args = args

        if not len(self.args[1:]) >= 2:
            self.__list_nouns()
            print >> sys.stderr, "Please give at least a noun and a verb."
            raise SystemExit(codes.system.NOT_ENOUGH_PARAMETERS)
        # Get all the options passed in
        self.noun, self.verb = args[1:3]

        # Generate the code based from the input
        action = __import__("%s.%s" % (self.plugin_package, self.noun),
                            globals(),
                            locals(),
                            [self.noun])
        self.action_to_run = action.__getattribute__(self.noun.capitalize())()
        self.options = self.parse_options()

    def __list_nouns(self):
        """
        Lists all available nouns.
        """
        action_mod = __import__(self.plugin_package)
        print >> sys.stderr, "Available nouns:",
        # Get the module path from __path__ of action_mod and plugin_package
        mod_path = os.path.join(action_mod.__path__[0],
                                self.plugin_package.split('.')[-1])
        # Go over each and print out the ones that are actions
        for noun in os.listdir(mod_path):
            if "__" not in noun and '.pyc' not in noun:
                print >> sys.stderr, "%s " % noun.replace('.py', ''),
        print >> sys.stderr, ""

    def parse_options(self):
        """
        Parse the options into something that can be passed to a method.

        Returns a usable dictionary to pass to a method.
        """
        parser = OptionParser()
        a_verb = self.action_to_run.__getattribute__(self.verb)
        # Since decorators change what inspect sees we try to use the original
        # method directly which gets attached as a_verb.meth. If it doesn't
        # exist then we try the method directly as it is either old style,
        # using direct method variables or has no help at all.
        try:
            inspection_data = [x for x in inspect.getargspec(a_verb.meth)]
        except:
            inspection_data = [x for x in inspect.getargspec(a_verb)]

        if inspection_data[0] == None:
            inspection_data[0] = []
        iargs = [x for x in reversed(inspection_data[0][1:])]
        if inspection_data[3] == None:
            inspection_data[3] = []
        iargs_defaults = [x for x in reversed(inspection_data[3])]

        defaults = {}
        for iarg_x in range(len(iargs)):
            # Default action is to store
            action = 'store'
            try:
                # Make sure we set up defaults
                def_item = iargs_defaults[iarg_x]
                if type(def_item) == types.TupleType:
                    def_item = def_item[0]
                defaults[iargs[iarg_x]] = def_item

                # Setup the action if it is a bool
                if defaults[iargs[iarg_x]] == True:
                    action = 'store_true'
                elif defaults[iargs[iarg_x]] == False:
                    action = 'store_false'
            except IndexError, ie:
                # Not everything has a default
                pass

            # Add it to optparse
            parser.add_option("--%s" % iargs[iarg_x],
                              dest=iargs[iarg_x],
                              action=action)

        # Bind the defaults
        parser.set_defaults(**defaults)
        options, largs = parser.parse_args(self.args[3:])
        return options.__dict__

    def run_code(self):
        """
        Takes care of running the code created.

        code is the code to execute.
        """
        self.action_to_run.__getattribute__(
            self.verb).__call__(**self.options)

    def run(self, filter_obj=None):
        """
        Runs the generated code.

        :Parameters:
            - `filter_obj`: the filter object.
        """
        try:
            self.run_code()
        except Exception, ex:
            # If we have a filters then use them ...
            if filter_obj:
                filter_obj.execute_filters(ex)
            else:
                # If we have no filters then raise the exception
                raise ex
            raise SystemExit(codes.system.EXCEPTION_RAISED)



# Copyright 2008, Red Hat, Inc
# Steve 'Ashcrow' Milner <smilner@redhat.com>
#
# This software may be freely redistributed under the terms of the GNU
# general public license.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
Test for director
"""

__docformat__ = 'restructuredtext'


import exceptions
import os
import sys
import tempfile
import unittest

from director.filter import Filter
from director.filter import ExceptionFilter


STDOUT = sys.stdout
STDERR = sys.stderr
TMP = tempfile.mkstemp()[1]


class ActionTests(unittest.TestCase):
    """
    Tests to verify that base action class works correctly.
    """

    def setUp(self):
        """
        Sets up stuff for the test.
        """
        import sys
        from director import Action

        # Remap stdout for these tests
        self.action = Action()
        sys.stdout = open(TMP, 'w')
        sys.stderr = open(TMP, 'w')

    def tearDown(self):
        """
        Tears down stuff for config test.
        """
        del self.action
        # Map the original stdout back
        sys.stdout.close()
        sys.stdout = STDOUT

    def test__list_verbs(self):
        """
        Tests the _list_verbs method.
        """
        self.assertEqual(self.action._list_verbs(), ['description', 'help'])

    def test__list_verbs(self):
        """
        Tests the _list_verbs method.
        """
        self.assertEqual(self.action._list_verbs(), ['description', 'help'])

    def test_description(self):
        """
        Tests the description works.
        """
        self.action.description()

    def test_help(self):
        """
        Tests the help works.
        """
        self.action.help()
        self.action.help('help')

    def test__action_help(self):
        """
        Tests the _action_help works.
        """
        self.action._action_help()


class ActionRunnerTests(unittest.TestCase):
    """
    Tests to verify base ActionRunner class works as expected.
    """

    def setUp(self):
        """
        Sets up stuff for the test.
        """
        from director import ActionRunner

        self.arunner = ActionRunner(['self', 'simpleaction',
                                     'verb', '--opt=value', '--another'],
                                    'tests.actions')

    def test_parse_options(self):
        """
        Make sure the parsing of options works.
        """
        res = self.arunner.parse_options()
        self.assertEqual(res, {'opt': 'value',
                               'another': False,
                               'last': 'last'})

    def test_run_code(self):
        """
        Verify generated code runs.
        """
        self.arunner.run_code()

    def test_run_with_filter(self):
        """
        Verify generated code runs with filter.
        """
        filter = Filter()
        filter.register_filter(ExceptionFilter(exceptions.IOError, ''))
        self.arunner.verb = 'verb'
        self.arunner.options = {'opt': 'ok'}
        self.arunner.run(filter)

    def test_run_without_filter(self):
        """
        Verify generated code runs without filter.
        """
        self.arunner.run()

    def test_run_execption_without_filter(self):
        """
        Verify generated raises an exception if there are no filters
        during run.
        """
        self.arunner.verb = "asdasd"
        self.assertRaises(AttributeError, self.arunner.run)



#!/usr/bin/env python
#
# Copyright 2008, Red Hat, Inc
# Steve 'Ashcrow' Milner <smilner@redhat.com>
#
# This software may be freely redistributed under the terms of the GNU
# general public license.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
Standard build script.
"""

__docformat__ = 'restructuredtext'


import os
import sys

from distutils.core import Command, setup
from glob import glob
from os.path import basename, walk, splitext
from os.path import join as pjoin
from os import path
from unittest import TextTestRunner, TestLoader

sys.path.insert(0, 'src')
sys.path.insert(1, 'tests')

from director import __version__, __license__, __author__


class SetupBuildCommand(Command):
    """
    Master setup build command to subclass from.
    """

    user_options = []

    def initialize_options(self):
        """
        Setup the current dir.
        """
        self._dir = os.getcwd()

    def finalize_options(self):
        """
        No clue ... but it's required.
        """
        pass


class RPMBuildCommand(SetupBuildCommand):
    """
    Creates an RPM based off spec files.
    """

    description = "Build an rpm based off of the top level spec file(s)"

    def run(self):
        """
        Run the RPMBuildCommand.
        """
        try:
            if os.system('./setup.py sdist'):
                raise Exception("Couldn't call ./setup.py sdist!")
                sys.exit(1)
            if not os.access('dist/rpms/', os.F_OK):
                os.mkdir('dist/rpms/')
            dist_path = os.path.join(os.getcwd(), 'dist')
            rpm_cmd = 'rpmbuild -ba --define "_rpmdir %s/rpms/" \
                                    --define "_srcrpmdir %s/rpms/" \
                                    --define "_sourcedir %s" *spec' % (
                      dist_path, dist_path, dist_path)
            if os.system(rpm_cmd):
                raise Exception("Could not create the rpms!")
        except Exception, ex:
            print >> sys.stderr, str(ex)


class SphinxCommand(SetupBuildCommand):
    """
    Creates HTML documentation using Sphinx.
    """

    description = "Generate documentation via sphinx"

    def run(self):
        """
        Run the DocCommand.
        """
        print "Creating html documentation ..."

        try:
            from sphinx.application import Sphinx

            if not os.access(path.join('docs', 'html'), os.F_OK):
                os.mkdir(path.join('docs', 'html'))
            buildername = 'html'
            outdir = path.abspath(path.join('docs', 'html'))
            doctreedir = os.path.join(outdir, '.doctrees')

            confdir = path.abspath('docs')
            srcdir = path.abspath('docs')
            freshenv = False

            # Create the builder
            app = Sphinx(srcdir, confdir, outdir, doctreedir, buildername,
                         {}, sys.stdout, sys.stderr, freshenv)

            # And build!
            app.builder.build_all()
            print "Your docs are now in %s" % outdir
        except ImportError, ie:
            print >> sys.stderr, "You don't seem to have the following which"
            print >> sys.stderr, "are required to make documentation:"
            print >> sys.stderr, "\tsphinx.application.Sphinx"
        except Exception, ex:
            print >> sys.stderr, "FAIL! exiting ... (%s)" % ex


class ViewDocCommand(SetupBuildCommand):
    """
    Quick command to view generated docs.
    """

    def run(self):
        """
        Opens a webbrowser on docs/html/index.html
        """
        import webbrowser

        print ("NOTE: If you have not created the docs first this will not be "
               "helpful. If you don't see any documentation in your browser "
               "run ./setup.py doc first.")
        if not webbrowser.open('docs/html/index.html'):
            print >> sys.stderr, "Could not open on your webbrowser."


class TestCommand(SetupBuildCommand):
    """
    Distutils testing command.
    """

    def run(self):
        """
        Finds all the tests modules in tests/, and runs them.
        """
        testfiles = []
        for t in glob(pjoin(self._dir, 'tests', '*.py')):
            if not t.endswith('__init__.py'):
                testfiles.append('.'.join(
                    ['tests', splitext(basename(t))[0]]))

        tests = TestLoader().loadTestsFromNames(testfiles)
        t = TextTestRunner(verbosity = 2)
        t.run(tests)


class TODOCommand(SetupBuildCommand):
    """
    Quick command to show code TODO's.
    """

    description = "prints out TODO's in the code"

    def run(self):
        """
        Prints out TODO's in the code.
        """
        import re

        format_str = "%s (%i): %s"

        # If a TODO exists, read it out
        try:
            line_no = 0
            todo_obj = open('TODO', 'r')
            for line in todo_obj.readlines():
                print format_str % ("TODO", line_no, line[:-1])
                line_no += 1
            todo_obj.close()
        except:
            pass

        remove_front_whitespace = re.compile("^[ ]*(.*)$")
        for rootdir in ['src/', 'bin/']:

            for root, dirs, files in os.walk(rootdir):
                for afile in files:
                    if afile[-4:] != '.pyc':
                        full_path = os.path.join(root, afile)
                        fobj = open(full_path, 'r')
                        line_no = 0
                        for line in fobj.readlines():
                            if 'todo' in line.lower():
                                nice_line = remove_front_whitespace.match(
                                    line).group(1)
                                print format_str % (full_path,
                                                       line_no,
                                                       nice_line)
                            line_no += 1


setup(name = "director",
    version = __version__,
    description = "Command line plugin library",
    long_description = """\
director is a python library that allows developers to create command line
plugins for tools making it easy to add new functionality.""",
    author = __author__,
    author_email = 'smilner+director@redhat.com',
    url = "https://fedorahosted.org/director/",
    download_url = "https://fedorahosted.org/releases/d/i/director/",
    platforms = ['any'],

    license = __license__,

    package_dir = {'director': 'src/director'},
    packages = ['director', 'director.codes'],

    classifiers = [
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Development Status :: 5 - Production/Stable',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Programming Language :: Python'],

    cmdclass = {'test': TestCommand,
                'doc': SphinxCommand,
                'viewdoc': ViewDocCommand,
                'rpm': RPMBuildCommand,
                'todo': TODOCommand})



# Copyright 2008, Red Hat, Inc
# Steve 'Ashcrow' Milner <smilner@redhat.com>
#
# This software may be freely redistributed under the terms of the GNU
# general public license.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
All decorators for director.
"""

__docformat__ = 'restructuredtext'


def simple_help(help_txt):
    """
    Adds a help variable to a class method as well as saving the original
    method as meth. This is best used for very simple help cases.

    :Parameters:
        - `help_txt`: the text to add as help text.
    """

    def decorator(meth):
        """
        Top level inner decorator which takes in a class method.

        :Parameters:
            - `meth`: the actual class method.
        """

        def wrapper(self, *args, **kwargs):
            """
            Internal wrapper that actually executes the method.

            :Parameters:
                - `self`: the class methods container object.
                - `*args`: any non keyword arguments.
                - `**kwargs`: all keyword arguments.
            """
            return meth(self, *args, **kwargs)

        # Add help variable and it's text
        wrapper.help = help_txt
        # Attach the original method as meth for inspection
        wrapper.meth = meth
        return wrapper

    return decorator


def general_help(desc, options={}, examples=[]):
    """
    Adds a help variable to a class method based on keyword arguments as well
    as saving the original method as meth. This can be used in more verbose
    help cases.

    :Parameters:
        - `desc`: the description.
        - `options`: a dictionary mapping options and their descriptions.
        - `examples`: a list of examples.
    """

    def decorator(meth):
        """
        Top level inner decorator which takes in a class method.

        :Parameters:
            - `meth`: the actual class method.
        """

        def wrapper(self, *args, **kwargs):
            """
            Internal wrapper that actually executes the method.

            :Parameters:
                - `self`: the class methods container object.
                - `*args`: any non keyword arguments.
                - `**kwargs`: all keyword arguments.
            """
            return meth(self, *args, **kwargs)

        # Put together the rendered help string
        rendered = "\n%s\n" % desc
        if options:
            rendered += "OPTIONS:"
            for key in options.keys():
                rendered += "\n\t%s\t%s" % (key, options[key])
        if examples:
            rendered += "\nEXAMPLES:"
            for example in examples:
                rendered += "\n\t%s" % example

        # Add help variable and it's text
        wrapper.help = rendered
        # Attach the original method as meth for inspection
        wrapper.meth = meth
        return wrapper

    return decorator
